; /**
;   ******************************************************************************
;   *  _ _ _    _ _ _   _ _
;   * |     \  /     |/     \
;   * |      \/           _ _\
;   * |                  /
;   * |         MultiClet
;   * |                  \ _ _
;   * |                      /
;   * |_ _ _|\/|_ _ _|\ _ _ /
;   *
;   * @file    gptimer_irq.asm 
;   * @author  Команда разработчиков Мультиклет
;   * @date    13-07-2015
;   * @brief   Пример работы с тамером общего назначения (GPTIM) и мигание светодиодом. 
;   *          
;   ******************************************************************************
;   * @attention
;   *
;   * <h2><center>&copy; COPYRIGHT(c) 2015 Multiclet corp.</center></h2>
;   *
;   *  ДАННЫЙ ПРИМЕР НАПИСАН ДЛЯ БЫСТРОГО ОЗНАКОМЛЕНИЯ ПОЛЬЗОВАТЕЛЯ С
;   *  ВОЗМОЖНОСТЯМИ МУЛЬТИКЛЕТОЧНОГО ПРОЦЕССОРА. КОМПАНИЯ МУЛЬТИКЛЕТ НЕ НЕСЁТ
;   *  ОТВЕТСТВЕННОСТИ ЗА ВОЗМОЖНЫЙ ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ПРИЧИНЁННЫЙ 
;   *  ИСПОЛЬЗОВАНИЕМ В ПОЛЬЗОВАТЕЛЬСКИХ ПРОДУКТАХ ИНФОРМАЦИИ, СОДЕРЖАЩЕЙСЯ
;   *  В ДАННОМ ПРИМЕРЕ.
;   * 
;   ******************************************************************************
;   */
;файл констант периферии для ассемблера
.include "R1_pcf.inc"

.text

;расположение светодиодов: 
;LED0 - GPIOB(1<<25), LED1 - GPIOB(1<<24), LED2 - GPIOB(1<<23)
;LED3 - GPIOB(1<<27), LED4 - GPIOC(1<<26), LED5 - GPIOB(1<<29)
;LED6 - GPIOB(1<<28), LED7 - GPIOA(1<<25)

;устанавливаем параграф - обработчик прерываний, маску прерываний
set_irq_handler:
    setl #IHOOKADDR, gptim0_irq_handler ;записываем метку параграфа для обработки прерываний
    setl #MSKR, 1<<29 ;записываем маску для разрешения прерываний от Таймера 0
    jmp set_gptim0 
complete

;устанавливаем период, предделитель, разрешение прерываний и непрерывный режим работы Таймера 0
set_gptim0:
    getl 0xFFFFF
    getl 0x4
    getl 0xF 
    wrl @3, TIM0_CNT0 ;записываем период Таймера 0
    wrl @3, TIM0_PSC ;записываем предделитель Таймера 0
    wrl @3, TIM0_CR0 ;записываем значение в регистр управления Таймера 0
    getl #PSW
    orl @1, 1
    setl #PSW, @1 ;разрешаем прерывания
    jmp config_port
complete

;сконфигурируем порт для LED1 на выход 
config_port:
    rdl GPIOB_DIR ;считываем текущее значение конфигурации порта B на вход и выход
    orl @1, (1<<24) ;формируем значение для включения 24-го разряда порта B на выход
    wrl @1, GPIOB_DIR ;записываем получившееся значение в регистр конфигурации порта B
    jmp wait1
complete

;простой цикл в ожидании прерывания
wait1:
    getl 123
    jmp wait1
complete

;обработчик прерываний от Таймера 0
gptim0_irq_handler:
    setl #INTR, 1<<29 ;снимаем прерывание от Таймера 0
    jmp led_blink
complete

;мигание светодиодом
led_blink:
    rdl GPIOB_OUT 
    xorl @1, (1<<24) 
    wrl @1, GPIOB_OUT 
    getl #PSW
    orl @1, 1
    setl #PSW, @1 ;разрешаем прерывания
    jmp #IRETADDR ;возвращаемся обратно, откуда ушли на прерывание
complete






  


