; /**
;   ******************************************************************************
;   *  _ _ _    _ _ _   _ _
;   * |     \  /     |/     \
;   * |      \/           _ _\
;   * |                  /
;   * |         MultiClet
;   * |                  \ _ _
;   * |                      /
;   * |_ _ _|\/|_ _ _|\ _ _ /
;   *
;   * @file    pll_led.asm 
;   * @author  Команда разработчиков Мультиклет
;   * @date    13-07-2015
;   * @brief   Пример работы с PLL, отключение сторожевого таймера и мигание светодиодом. 
;   *          
;   ******************************************************************************
;   * @attention
;   *
;   * <h2><center>&copy; COPYRIGHT(c) 2015 Multiclet corp.</center></h2>
;   *
;   *  ДАННЫЙ ПРИМЕР НАПИСАН ДЛЯ БЫСТРОГО ОЗНАКОМЛЕНИЯ ПОЛЬЗОВАТЕЛЯ С
;   *  ВОЗМОЖНОСТЯМИ МУЛЬТИКЛЕТОЧНОГО ПРОЦЕССОРА. КОМПАНИЯ МУЛЬТИКЛЕТ НЕ НЕСЁТ
;   *  ОТВЕТСТВЕННОСТИ ЗА ВОЗМОЖНЫЙ ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ПРИЧИНЁННЫЙ 
;   *  ИСПОЛЬЗОВАНИЕМ В ПОЛЬЗОВАТЕЛЬСКИХ ПРОДУКТАХ ИНФОРМАЦИИ, СОДЕРЖАЩЕЙСЯ
;   *  В ДАННОМ ПРИМЕРЕ.
;   * 
;   ******************************************************************************
;   */
;файл констант периферии для ассемблера
.include "R1_pcf.inc"

.text

;расположение светодиодов: 
;LED0 - GPIOB(1<<25), LED1 - GPIOB(1<<24), LED2 - GPIOB(1<<23)
;LED3 - GPIOB(1<<27), LED4 - GPIOC(1<<26), LED5 - GPIOB(1<<29)
;LED6 - GPIOB(1<<28), LED7 - GPIOA(1<<25)

;регистры WDT:
;WDT_CNT - значение периода счетчика(после снятия сигнала сброс устанавливается 0xFFFFFFFF)
;WDT_KEY - регистр переключения режимов (0x3333-выключить WDT)
;WDT_ST - регистр состояния

;регистры PLL:
;PLLCR - регистр управления генератора
;PLLSTR - регистр состояния генератора
;PLLTMCR - регистр управления модулем коммутации

;настройка частоты PLL:
;1) установить биты NCNT и RCNT в регистре PLLCR
;2) установить бит EN в регистре PLLCR в "1"
;3) дождаться установки бита LOCK в регистре PLLSTR
;4) установить биты MXVCO в регистре PLLCR
;время переключения после пункта 4 составляет: Период_исходной_частоты + 2*Период_новой_частоты
;процессор R1 статический, поэтому переключение частоты не влияет на его работу

;отключение сторожевого таймера (по умолчанию он всегда включен в процессоре)
wdt_off:
	getl 0x00003333 ;загружаем в коммутатор значение
    wrl @1, WDT_KEY ;записываем это значение для отключения WDT
    jmp pll_100 ;переходим на настройку PLL
complete

;переход на частоту 100МГц (с учетом, что на плате установлен кварцевый резонатор на 8 МГц)///
pll_100:
    getl 0xC02E ;формируем значение для регистра PLLCR (NCNT+RCNT+EN)
    setl #PLLCR, @1 ;устанавливаем значение в регистр управления
    jmp pll_100_lock 
complete

pll_100_lock:
    getl #PLLSTR ;считываем текущее значение статусного регистра
    getl 1 
    andl @1, @2 ;анализируем 0-й разряд
    je @1, pll_100_lock ;если бит не установлен в "1", то остаемся в данном параграфе
    jne @2, pll_100_set ;если бит установлен в "1", то переходим далее
complete

pll_100_set:
    getl 0xCC2E ;формируем значение для регистра PLLCR (MXVCO)
    setl #PLLCR, @1 ;устанавливаем значение в регистр управления
    jmp config_port ;переходим на управление светодиодами
complete
;///через 14 тактов будет установлена частота 100МГц

;сконфигурируем порт для LED1 на выход 
config_port:
    rdl GPIOB_DIR ;считываем текущее значение конфигурации порта B на вход и выход
    orl @1, (1<<24) ;формируем значение для включения 24-го разряда порта B на выход
    wrl @1, GPIOB_DIR ;записываем получившееся значение в регистр конфигурации порта B
    setl #0, 1000000 ;для формирования задержки установим счетчик в виде РОНа 0
    jmp wait1 ;осуществляем переход на параграф с задержкой
complete

;задержка
wait1:
    getl #0 ;получаем значение РОНа 0
    subl @1, 1 ;вычитаем единицу
    je @1, led1_on ;проверяем когда счетчик станет равен нулю
    jne @2, wait1 ;если счетчик не равне нулю, то остаемся в этом параграфе
    setl #0, @3 ;записываем в РОН 0
complete

;включение светодиода
led1_on:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    orl @1, (1<<24) ;формируем значение для включения логической единицы на 24-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
    setl #0, 1000000 ;для формирования задержки установим счетчик в виде РОНа 0
    jmp wait2 ;осуществляем переход на параграф с задержкой
complete

;задержка
wait2:
    getl #0 ;получаем значение РОНа 0
    subl @1, 1 ;вычитаем единицу
    je @1, led1_off ;проверяем когда счетчик станет равен нулю
    jne @2, wait2 ;если счетчик не равне нулю, то остаемся в этом параграфе
    setl #0, @3 ;записываем в РОН 0
complete

;выключение светодиода
led1_off:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    andl @1, ~(1<<24) ;формируем значение для выключения логической единицы на 24-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
    setl #0, 1000000 ;для формирования задержки установим счетчик в виде РОНа 0
    jmp wait1 ;осуществляем переход на параграф с задержкой
complete






  


