/****************************************Copyright (c)**************************************************                         
**
**                                 http://www.powermcu.com
**
**--------------File Info-------------------------------------------------------------------------------
** File name:			GLCD.c
** Descriptions:		Have been tested HY28A-LCDB HY28B
**						
**------------------------------------------------------------------------------------------------------
** Created by:			AVRman
** Created date:		2013-12-4
** Version:				2.1
** Descriptions:		The original version
**
**------------------------------------------------------------------------------------------------------
** Modified by:			Multiclet team
** Modified date:	
** Version:
** Descriptions:		
********************************************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "GLCD.h"
#include "AsciiLib.h"

unsigned short DeviceCode;
	
/*******************************************************************************
* Function Name  : LCD_Delay
* Description    : Delay Time
* Input          : - nCount: Delay Time
* Output         : None
* Return         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void delay_ms( unsigned short ms)    
{ 
	unsigned short i,j,k; 
	for(i=0;i<ms;i++){		
		for(j=0;j<0xAD0;j++);
	}
}

/*******************************************************************************
* Function Name  : Lcd_Configuration
* Description    : Configures LCD Control lines
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Configuration(void)
{
	/*Disable SPI*/	
	*((uint32_t*)0xC0104020)=(*((uint32_t*)0xC0104020))&(~0x1000000);
	
 	/*Puts 16 bit bus to GND*/
 	GPIOA->BPS &= ~0x3A000000;
	GPIOA->DIR |= 0x3A000000;
	GPIOA->OUT &= ~0x3A000000;	

	GPIOB->BPS &= ~0x3C00000;
	GPIOB->DIR |=  0x3C00000;
	GPIOB->OUT &= ~0x3C00000;

	GPIOD->BPS &= ~0x1FFE;
	GPIOD->DIR |=  0xFFE;
	GPIOD->OUT  = ((GPIOD->OUT&(0xDFE))|0x200);

 	
 	/*Conf SPI pins*/
 	GPIOC->BPS = GPIOC->BPS|0x7;
	
	/*CS*/
	GPIOD->BPS &= ~0x6000;
	GPIOD->DIR |=  0x6000;
	GPIOD->OUT &=  ~0x2000;
		
	/*Conf SPI*/
	*((uint32_t*)0xC0104038)=1;
	*((uint32_t*)0xC0104020)=/*0x27732085;*/0x37732085;/*0x37702085;*/
	/*Enable SPI*/	
	*((uint32_t*)0xC0104020)=(*((uint32_t*)0xC0104020))|0x1000000;
	delay_ms(10);
	
	GPIOF->BPS &=  ~0x380E0000;
	GPIOF->DIR |=  0x38000000|0x100000|0xC0000;
	GPIOF->DIR &=  ~0x20000;
	GPIOF->OUT &=  (~0x80C0000);
	GPIOF->OUT |=  0x30000000|0x100000;
}

/*******************************************************************************
* Function Name  : LPC17xx_SPI_SendRecvByte
* Description    : Send one byte then recv one byte of response
* Input          : - byte_s: byte_s
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
unsigned char LPC17xx_SPI_SendRecvByte (unsigned char byte_s)
{
	int i;
	while (((*((uint32_t*)0xC0104024))&0x100)==0);

	*((uint32_t*)0xC0104030)=byte_s<<24;
	while (((*((uint32_t*)0xC0104024))&0x80000000));

	while (((*((uint32_t*)0xC0104024))&0x200)==0);

	return (*((uint32_t*)0xC0104034))>>16;
}
void LPC17xx_SPI_SendByte (unsigned char byte_s)
{
	while (((*((uint32_t*)0xC0104024))&0x100)==0);
	*((uint32_t*)0xC0104030)=byte_s<<24;
	while (((*((uint32_t*)0xC0104024))&0x80000000));
}

/*******************************************************************************
* Function Name  : LCD_WriteIndex
* Description    : LCD write register address
* Input          : - index: register address
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_WriteIndex(unsigned char index)
{
	SPI_CS_LOW;

        /* SPI write data */
	LPC17xx_SPI_SendByte(SPI_START | SPI_WR | SPI_INDEX);   /* Write : RS = 0, RW = 0       */
	LPC17xx_SPI_SendByte(0);
	LPC17xx_SPI_SendByte(index);

	SPI_CS_HIGH; 
}

/*******************************************************************************
* Function Name  : LCD_WriteData
* Description    : LCD write register data
* Input          : - data: register data
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_WriteData( unsigned short data)
{
	SPI_CS_LOW;

	LPC17xx_SPI_SendByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
	LPC17xx_SPI_SendByte((data >>   8));                    /* Write D8..D15                */
	LPC17xx_SPI_SendByte((data & 0xFF));                    /* Write D0..D7                 */

	SPI_CS_HIGH; 
}

/*******************************************************************************
* Function Name  : LCD_Write_Data_Start
* Description    : Start of data writing to the LCD controller
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Write_Data_Start(void) 
{
  LPC17xx_SPI_SendByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
}

/*******************************************************************************
* Function Name  : LCD_Write_Data_Only
* Description    : Data writing to the LCD controller
* Input          : - data: data to be written  
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Write_Data_Only( unsigned short data) 
{
  LPC17xx_SPI_SendByte((data >>   8));      /* Write D8..D15                */
  LPC17xx_SPI_SendByte((data & 0xFF));      /* Write D0..D7                 */
}

/*******************************************************************************
* Function Name  : LCD_ReadData
* Description    : LCD read data
* Input          : None
* Output         : None
* Return         : return data
* Attention	 : None
*******************************************************************************/
unsigned short LCD_ReadData(void)
{ 
	unsigned short value;
	SPI_CS_LOW;

	LPC17xx_SPI_SendRecvByte(SPI_START | SPI_RD | SPI_DATA);    
	LPC17xx_SPI_SendRecvByte(0);                                
	value   = LPC17xx_SPI_SendRecvByte(0);                      
	value <<= 8;
	value  |= LPC17xx_SPI_SendRecvByte(0);                     

	SPI_CS_HIGH; 
	
	return value;
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Writes to the selected LCD register.
* Input          : - LCD_Reg: address of the selected register.
*                  - LCD_RegValue: value to write to the selected register.
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_WriteReg( unsigned short LCD_Reg, unsigned short LCD_RegValue)
{
	/* Write 16-bit Index, then Write Reg */  
	LCD_WriteIndex(LCD_Reg);         
	/* Write 16-bit Reg */
	LCD_WriteData(LCD_RegValue);  
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Reads the selected LCD Register.
* Input          : None
* Output         : None
* Return         : LCD Register Value.
* Attention		 : None
*******************************************************************************/
unsigned short LCD_ReadReg( unsigned short LCD_Reg)
{
	unsigned short LCD_RAM;
	
	/* Write 16-bit Index (then Read Reg) */
	LCD_WriteIndex(LCD_Reg);
	/* Read 16-bit Reg */
	LCD_RAM = LCD_ReadData();
    	
	return LCD_RAM;
}

/*******************************************************************************
* Function Name  : LCD_SetCursor
* Description    : Sets the cursor position.
* Input          : - Xpos: specifies the X position.
*                  - Ypos: specifies the Y position. 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
static void LCD_SetCursor( unsigned short Xpos, unsigned short Ypos )
{
	LCD_WriteReg(0x0020, Xpos );     
	LCD_WriteReg(0x0021, Ypos );           
}


/*******************************************************************************
* Function Name  : LCD_Initializtion
* Description    : Initialize TFT Controller.
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Initializtion(void)
{
	
	LCD_Configuration();	
		
        DeviceCode = LCD_ReadReg(0x0000);

	LCD_WriteReg(0x00e7,0x0010);      
	LCD_WriteReg(0x0000,0x0001);  	/* start internal osc */
	LCD_WriteReg(0x0001,0x0100);     
	LCD_WriteReg(0x0002,0x0700); 	/* power on sequence */
	LCD_WriteReg(0x0003,0x18/*(1<<12)|(1<<5)|(1<<4)|(0<<3)*/ ); 	/* importance */
	LCD_WriteReg(0x0004,0x0000);                                   
	LCD_WriteReg(0x0008,0x0207);	           
	LCD_WriteReg(0x0009,0x0000);         
	LCD_WriteReg(0x000a,0x0000); 	/* display setting */        
	LCD_WriteReg(0x000c,0x0001);	/* display setting */        
	LCD_WriteReg(0x000d,0x0000); 			        
	LCD_WriteReg(0x000f,0x0000);
	/* Power On sequence */
	LCD_WriteReg(0x0010,0x0000);   
	LCD_WriteReg(0x0011,0x0007);
	LCD_WriteReg(0x0012,0x0000);                                                                 
	LCD_WriteReg(0x0013,0x0000);                 
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0010,0x1590);   
	LCD_WriteReg(0x0011,0x0227);
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0012,0x009c);                  
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0013,0x1900);   
	LCD_WriteReg(0x0029,0x0023);
	LCD_WriteReg(0x002b,0x000e);
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0020,0x0000);                                                            
	LCD_WriteReg(0x0021,0x0000);           
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0030,0x0007); 
	LCD_WriteReg(0x0031,0x0707);   
	LCD_WriteReg(0x0032,0x0006);
	LCD_WriteReg(0x0035,0x0704);
	LCD_WriteReg(0x0036,0x1f04); 
	LCD_WriteReg(0x0037,0x0004);
	LCD_WriteReg(0x0038,0x0000);        
	LCD_WriteReg(0x0039,0x0706);     
	LCD_WriteReg(0x003c,0x0701);
	LCD_WriteReg(0x003d,0x000f);
	delay_ms(50);  /* delay 50 ms */		
	LCD_WriteReg(0x0050,0x0000);        
	LCD_WriteReg(0x0051,0x00ef);   
	LCD_WriteReg(0x0052,0x0000);     
	LCD_WriteReg(0x0053,0x013f);
	LCD_WriteReg(0x0060,0xa700);        
	LCD_WriteReg(0x0061,0x0001); 
	LCD_WriteReg(0x006a,0x0000);
	LCD_WriteReg(0x0080,0x0000);
	LCD_WriteReg(0x0081,0x0000);
	LCD_WriteReg(0x0082,0x0000);
	LCD_WriteReg(0x0083,0x0000);
	LCD_WriteReg(0x0084,0x0000);
	LCD_WriteReg(0x0085,0x0000);
	  
	LCD_WriteReg(0x0090,0x0010);     
	LCD_WriteReg(0x0092,0x0000);  
	LCD_WriteReg(0x0093,0x0003);
	LCD_WriteReg(0x0095,0x0110);
	LCD_WriteReg(0x0097,0x0000);        
	LCD_WriteReg(0x0098,0x0000);  
	/* display on sequence */    
	LCD_WriteReg(0x0007,0x0133);
	
	LCD_WriteReg(0x0020,0x0000);         
	LCD_WriteReg(0x0021,0x0000);	
    delay_ms(50);   /* delay 50 ms */		
}

/*******************************************************************************
* Function Name  : LCD_Clear
* Description    : Fill the screen as the specified color
* Input          : - Color: Screen Color
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/

void LCD_DrawReactangle(uint16_t x,uint16_t y,uint16_t hight,uint16_t width,uint16_t Color){
	uint32_t cx,cy,bx;
	uint32_t MSB=Color<<16,LSB=Color<<24;
	x=x+hight-1;
	for(cy=0;cy<width;cy++){
		//cx=x;
		LCD_SetCursor(y,x); 
		LCD_WriteIndex(0x0022);
		SPI_CS_LOW; 
		LCD_Write_Data_Start();
		for(bx=0;bx<hight;bx++){			
			//while (((*((uint32_t*)0xC0104024))&0x100)==0);
			while (((*((uint32_t*)0xC0104024))&0x80000000));
			*((uint32_t*)0xC0104030)=MSB;
			//while (((*((uint32_t*)0xC0104024))&0x100)==0);
			while (((*((uint32_t*)0xC0104024))&0x80000000));
			*((uint32_t*)0xC0104030)=LSB;	
		}
		while (((*((uint32_t*)0xC0104024))&0x80000000));
		SPI_CS_HIGH;
		y++;
	}
}

void LCD_Clear( unsigned short Color)
{
	unsigned int index=0;
	uint32_t MSB=Color<<16,LSB=Color<<24;
	
	LCD_SetCursor(0,0); 

	LCD_WriteIndex(0x0022);

 	SPI_CS_LOW; 
	LCD_Write_Data_Start();

	for( index = 0; index < MAX_XY; index+=1 ){
		while (((*((uint32_t*)0xC0104024))&0x100)==0);
		*((uint32_t*)0xC0104030)=MSB;
		while (((*((uint32_t*)0xC0104024))&0x100)==0);
		*((uint32_t*)0xC0104030)=LSB;	
	}	
	while (((*((uint32_t*)0xC0104024))&0x80000000));
	SPI_CS_HIGH;
}

/******************************************************************************
* Function Name  : LCD_SetPoint
* Description    : Drawn at a specified point coordinates
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_SetPoint( unsigned short Xpos, unsigned short Ypos, unsigned short point)
{
	LCD_SetCursor(Xpos,Ypos);
	LCD_WriteReg(0x0022,point);
}

/******************************************************************************
* Function Name  : LCD_DrawLine
* Description    : Bresenham's line algorithm
* Input          : - x1: A point line coordinates
*                  - y1: A point column coordinates 
*		   - x2: B point line coordinates
*		   - y2: B point column coordinates 
*		   - color: Line color
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/	 
void LCD_DrawLine( uint16_t y0, uint16_t x0, uint16_t y1, uint16_t x1 , uint16_t color )
{
    short dx,dy;      /* The definition of the X Y axis increase the value of the variable */
    short temp;       

    if( x0 > x1 )     
    {
	    temp = x1;
		x1 = x0;
		x0 = temp;   
    }
    if( y0 > y1 )     
    {
		temp = y1;
		y1 = y0;
		y0 = temp;   
    }
  
	dx = x1-x0;       
	dy = y1-y0;       

    if( dx == 0 )     
    {
        do
        { 
            LCD_SetPoint(x0, y0, color);   
            y0++;
        }
        while( y1 >= y0 ); 
		return; 
    }
    if( dy == 0 )     
    {
        do
        {
            LCD_SetPoint(x0, y0, color);
            x0++;
        }
        while( x1 >= x0 ); 
		return;
    }
	
    if( dx > dy )                        
    {
	    temp = 2 * dy - dx;                
        while( x0 != x1 )
        {
	        LCD_SetPoint(x0,y0,color);  
	        x0++;                        
	        if( temp > 0 )               
	        {
	            y0++;                    
	            temp += 2 * dy - 2 * dx; 
	 	    }
            else         
            {
			    temp += 2 * dy;          
			}       
        }
        LCD_SetPoint(x0,y0,color);
    }  
    else
    {
	    temp = 2 * dx - dy;                     
        while( y0 != y1 )
        {
	 	    LCD_SetPoint(x0,y0,color);     
            y0++;                 
            if( temp > 0 )           
            {
                x0++;               
                temp+=2*dy-2*dx; 
            }
            else
			{
                temp += 2 * dy;
			}
        } 
        LCD_SetPoint(x0,y0,color);
	}
} 


/******************************************************************************
* Function Name  : PutChar
* Description    : Lcd screen displays a character
* Input          : - Xpos: Horizontal coordinate
*                  - Ypos: Vertical coordinate 
*				   - ASCI: Displayed character
*				   - charColor: Character color  
*				   - bkColor: Background color 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void PutChar( uint16_t Xpos, uint16_t Ypos, uint8_t ASCI, uint16_t charColor, uint32_t bkColor,int Big)
{
	uint16_t i, j,ii,jj;
    uint8_t buffer[16], tmp_char;
    GetASCIICode(buffer,ASCI);  /* get font data */
    for( i=0; i<15/*16*/; i++ )
    {
        tmp_char = buffer[i];
        for( j=0; j<8; j++ )
        {
            if( (tmp_char >> 7 - j) & 0x01 == 0x01 )
            {
				for(ii=0;ii<((Big!=0)?2:1);ii++){
					for(jj=0;jj<((Big!=0)?2:1);jj++)
						LCD_SetPoint( Xpos + ((Big!=0)?j*2:j)+jj, Ypos + ((Big!=0)?i*2:i)+ii, charColor );  /* Character color */
                }
            }
            else
            {
				if(bkColor!=0xFFFFFFFF){
					for(ii=0;ii<((Big!=0)?2:1);ii++){
					for(jj=0;jj<((Big!=0)?2:1);jj++)
						LCD_SetPoint( Xpos + ((Big!=0)?j*2:j)+jj, Ypos + ((Big!=0)?i*2:i)+ii, bkColor );  /* Character color */
					}
				}
            }
        }
    }
}

/******************************************************************************
* Function Name  : GUI_Text
* Description    : Displays the string
* Input          : - Xpos: Horizontal coordinate
*                  - Ypos: Vertical coordinate 
*				   - str: Displayed string
*				   - charColor: Character color   
*				   - bkColor: Background color  
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void GUI_Text(uint16_t Xpos, uint16_t Ypos, uint8_t *str,uint16_t Color, uint32_t bkColor,int Big)
{
    uint8_t TempChar;
    do
    {
        TempChar = *str++;  
        PutChar( Xpos, Ypos, TempChar, Color, bkColor,Big); 
		   
        if( Xpos < MAX_X - 8 )
        {
            Xpos += ((Big!=0)?16:8);
        } 
        else if ( Ypos < MAX_Y - 16 )
        {
            Xpos = 0;
            Ypos += 16;
        }   
        else
        {
            Xpos = 0;
            Ypos = 0;
        }    
    }
    while ( *str != 0 );
}

/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
