
//==============================================================================//
// Modbus Interface																//
//==============================================================================//
#ifndef __ModbusInterface_h__
#define __ModbusInterface_h__

#include <uart.h>
#include <gpio.h>
#include <irq.h>
#include <regs.h>
#include <tim.h>

#ifndef DWORD
#define DWORD unsigned
#endif //DWORD

#ifndef BYTE
#define BYTE char
#endif //BYTE

#define MB_INTERFACE_MODE_SLAVE   0
#define MB_INTERFACE_MODE_MASTER  1

enum
{
  MB_INTERFACE_STATE_NOT_CONFIGURED = 0,
  MB_INTERFACE_STATE_FREE,
  MB_INTERFACE_STATE_RECEIVE,
  MB_INTERFACE_STATE_RECEIVE_ERROR,
  MB_INTERFACE_STATE_RECEIVED,
  MB_INTERFACE_STATE_WAIT_ANSWER,
  MB_INTERFACE_STATE_SEND,
  MB_INTERFACE_STATE_SENT
};

#define MB_MAX_BUFFERSIZE  264

struct sUart;
typedef struct sUart Uart;

typedef void (*MbInterfaceHandler)(Uart *uart); // Callback для обработки пакетов

typedef struct
{
  DWORD Interface;       // номер интерфейса
  MbInterfaceHandler Handler; // Обработчик событий
  DWORD BuffSize;        // размер буфера
  BYTE  *Buff;           // указатель на рабочий на буфер
  DWORD MessageLength;   // длина сообщения
  DWORD Mode;            // режим работы (MB_INTERFACE_MODE_...)
  DWORD State;           // текущее состояние (MB_INTERFACE_STATE_...)
} MbInterface;

#endif //__ModbusInterface_h__



//------------------------------------------------------------------------------
// Структура уарта
//----
struct sUart
{
  MbInterface MbI;
  UART_TypeDef * Base; // базовый адрес Uart
  DWORD Baud;          // скорость
  DWORD Parity;        // паритет: 0 - NO, 1 - EVEV, 2 - ODD, 3 - SPACE, 4 - MARK
  GPIO_TypeDef * DriverEnableBitPort; // Порт для сигнала tx_en
  DWORD DriverEnableBitMask; // Порт для сигнала tx_en
  BYTE  *SendBuff; // не требующий инициализации служебный указатель на следующий передаваемый байт
};

//------------------------------------------------------------------------------
//  void UartInit(Uart *uart, DWORD priority)
//  Инициализация уарта
//  Input:
//    uart     -  структура уарта
//    priority -  приоритет обработчика (В настоящее время не используется
//				 в связи с отсутствием вложенных прерываний. Все прерывания обрабатываются в порядке поступления.)
//  Output: none
//----
void UartInit(Uart *uart, DWORD priority);

//------------------------------------------------------------------------------
//  void TimInit(Uart *uart)
//  Инициализировать таймер GPTIM0
//  Input:
//    uart     -  структура уарта
//  Output: none
//----
void TimInit();

//------------------------------------------------------------------------------
//  void UartDisableRecv(DWORD base)
//  Выключить прием
//  Input:
//    uart     -  структура уарта
//  Output: none
//----
void UartDisableRecv(Uart *uart);

//------------------------------------------------------------------------------
//  void UartDisableSend(DWORD base)
//  Выключить передачу
//  Input:
//    uart     -  структура уарта
//  Output: none
//----
void UartDisableSend(Uart *uart);

//------------------------------------------------------------------------------
//  void UartRecv(Uart *uart)
//  Включить прием
//  Input:
//    uart     -  структура уарта
//  Output: none
//----
void UartRecv(Uart * uart);

//------------------------------------------------------------------------------
//  void UartSend(Uart *uart)
//  Включить передачу
//  Input:
//    uart     -  структура уарта
//  Output: none
//----
void UartSend(Uart * uart);
