//PLL constant file
#ifndef PLL_H
#define PLL_H

#include "R1_ccf.h"

// для кварца 8 МГц переход на 100 МГц
#define PLL_8_100_RCNT 0x4
#define PLL_8_100_NCNT 0x2E
#define PLL_8_100_MXVCO 0xC




/**
  * @brief  Переход на новую частоту работы процессора с помощью блока PLL.
  * @param  RCNT: коэффициент делителя опорной частоты.
  * Данный параметр может принимать одно из следующих значений: 0-7.
  * @param  NCNT: коэффициент делителя в петле обратной связи.
  * Данный параметр может принимать одно из следующих значений: 0-127.
  * @param  MXVCO: коэффициент мультиплексора выходного сигнала.
  * Данный параметр может принимать одно из следующих значений: 0-15
  * @param  Enable: разрешение работы блока PLL (1-разрешено, 0 - запрещено).
  * Данный параметр может принимать одно из следующих значений: 0,1.
  * @retval 
  * 0 - работа pll разрешена, 1 - работа pll запрещена.
  */ 
  
int mc_set_pll(unsigned char RCNT, unsigned char NCNT, unsigned char MXVCO, unsigned char Enable);

/**
  * @brief  Переход на новую частоту работы процессора с помощью блока PLL.
  * @param  Freq : значение устанавливаемой частоты тактирования процессора.
  * @param  Sysclk : частота внешнего кварцевого генератора, тактирующего процессор.
  * Данный параметр может принимать одно из следующих значений:
  * 8,9,10,11,12,13,14,15.
  * @retval 
  * Нет
  */ 
int mc_get_freq(unsigned int xtal_freq); 

#endif















