#include "platform.h"
#include "tim.h"
#include "pll.h"

/**
  * @brief  Функция задержки в микросекундах.
  * @param  count : интервал задержки в микросекундах. 
  *   Данный параметр может принимать одно из следующих значений:
  *   0-28633115.
  * @retval Нет
  */
void mc_delay_us(unsigned int count)
{
    unsigned int res_delay;
    //Запускаем таймер, заданный пользователем (0-й канал)
	TIM_PSCPER(4, MC_TIMER_LIB);
	TIM_CNT(0xFFFFFFFF, MC_TIMER_LIB, 0); 
	TIM_CR(1, 0, 1, 0, 0, MC_TIMER_LIB, 0);
    //Рассчитываем необходимую задержку
    res_delay = 0xFFFFFFFF - (count * mc_get_freq(MC_XTAL_FREQ))/5;
    while(MC_TIMER_LIB->CNTVAL0 > res_delay);
    TIM_CR(0, 0, 0, 0, 0, MC_TIMER_LIB, 0);
}

/**
  * @brief  Функция задержки в миллисекундах.
  * @param  count : интервал задержки в микросекундах. 
  *   Данный параметр может принимать одно из следующих значений:
  *   0-42949.
  * @retval Нет
  */
void mc_delay_ms(unsigned int count)
{
    unsigned int res_delay;
    //Запускаем таймер, заданный пользователем (0-й канал)
	TIM_PSCPER(4, MC_TIMER_LIB);
	TIM_CNT(0xFFFFFFFF, MC_TIMER_LIB, 0); 
	TIM_CR(1, 0, 1, 0, 0, MC_TIMER_LIB, 0);
    //Рассчитываем необходимую задержку
    res_delay = 0xFFFFFFFF - (count * 200 * mc_get_freq(MC_XTAL_FREQ));
    while(MC_TIMER_LIB->CNTVAL0 > res_delay);
    TIM_CR(0, 0, 0, 0, 0, MC_TIMER_LIB, 0);
}

/**
  * @brief  Функция запуска таймера для выработки тиков в миллисекундах.
  * @param Нет
  * @retval Нет
  */
void mc_start_tick()
{
    //Запускаем таймер, заданный пользователем (1-й канал)
	TIM_PSCPER(4, MC_TIMER_LIB);
	TIM_CNT(0xFFFFFFFF, MC_TIMER_LIB, 1); 
	TIM_CR(1, 0, 1, 0, 0, MC_TIMER_LIB, 1);
}

/**
  * @brief  Функция остановки таймера для выработки тиков в миллисекундах.
  * @param  Нет
  * @retval Нет
  */
void mc_stop_tick()
{
    //Останавливаем таймер, заданный пользователем (1-й канал)
	TIM_CR(0, 0, 0, 0, 0, MC_TIMER_LIB, 1);
}

/**
  * @brief  Функция для получения количества тиков в миллисекундах.
  * @param  Нет
  * @retval Количество тиков в миллисекундах
  */
int mc_get_ms_tick()
{
    int uTick;
    //Получаем количество тиков
	uTick = ((0xFFFFFFFF - MC_TIMER_LIB->CNTVAL1)) / (200 * mc_get_freq(MC_XTAL_FREQ));
    return uTick;
}










